#!/usr/bin/env python3

from cgi import print_environ_usage
import cv2
import depthai as dai

# Otherwise (False), the aligned depth is automatically upscaled to 1080p
downscaleColor = True
fps = 30

stepSize = 0.05

newConfig = False

# Closer-in minimum depth, disparity range is doubled (from 95 to 190):
extended_disparity = False
# Better accuracy for longer distance, fractional disparity 32-levels:
subpixel = False
# Better handling for occlusions:
lr_check = False


MedianFilter = {
    "0": dai.MedianFilter.MEDIAN_OFF,
    "3": dai.MedianFilter.KERNEL_3x3,
    "5": dai.MedianFilter.KERNEL_5x5,
    "7": dai.MedianFilter.KERNEL_7x7,
}

median_filter = dai.MedianFilter.MEDIAN_OFF


# Create pipeline
pipeline = dai.Pipeline()

# Properties
camRgb = pipeline.create(dai.node.ColorCamera)
camRgb.setBoardSocket(dai.CameraBoardSocket.RGB)
camRgb.setResolution(dai.ColorCameraProperties.SensorResolution.THE_4_K)
camRgb.setFps(fps)
if downscaleColor:
    camRgb.setIspScale(1, 3)

# Define sources and outputs
monoLeft = pipeline.create(dai.node.MonoCamera)
monoRight = pipeline.create(dai.node.MonoCamera)
stereo = pipeline.create(dai.node.StereoDepth)
spatialLocationCalculator = pipeline.create(dai.node.SpatialLocationCalculator)

xoutDepth = pipeline.create(dai.node.XLinkOut)
xoutRgb = pipeline.create(dai.node.XLinkOut)
xoutSpatialData = pipeline.create(dai.node.XLinkOut)
xoutDepthConfig = pipeline.create(dai.node.XLinkIn)

xoutDepth.setStreamName("depth")
xoutRgb.setStreamName("rgb")
xoutSpatialData.setStreamName("spatialData")
xoutDepthConfig.setStreamName("DepthConfig")

# Properties
monoLeft.setResolution(dai.MonoCameraProperties.SensorResolution.THE_720_P)
monoLeft.setBoardSocket(dai.CameraBoardSocket.LEFT)
monoRight.setResolution(dai.MonoCameraProperties.SensorResolution.THE_720_P)
monoRight.setBoardSocket(dai.CameraBoardSocket.RIGHT)

# Create a node that will produce the depth map (using disparity output as it's easier to visualize depth this way)
stereo.setDefaultProfilePreset(dai.node.StereoDepth.PresetMode.HIGH_DENSITY)

#stereo.initialConfig.setConfidenceThreshold(255)
stereo.initialConfig.setLeftRightCheck(lr_check)
stereo.initialConfig.setExtendedDisparity(extended_disparity)
stereo.initialConfig.setSubpixel(subpixel)
stereo.initialConfig.setMedianFilter(median_filter)

config = stereo.initialConfig.get()
config.postProcessing.speckleFilter.enable = False
config.postProcessing.speckleFilter.speckleRange = 50
config.postProcessing.temporalFilter.enable = True
config.postProcessing.spatialFilter.enable = True
config.postProcessing.spatialFilter.holeFillingRadius = 2
config.postProcessing.spatialFilter.numIterations = 1
config.postProcessing.thresholdFilter.minRange = 400
config.postProcessing.thresholdFilter.maxRange = 15000
config.postProcessing.decimationFilter.decimationFactor = 1
stereo.initialConfig.set(config)

# stereo.setDepthAlign(dai.CameraBoardSocket.RGB)

spatialLocationCalculator.inputConfig.setWaitForMessage(False)

# config_top
topLeft = dai.Point2f(0.45, 0.25)
bottomRight = dai.Point2f(0.55, 0.35)

config_top = dai.SpatialLocationCalculatorConfigData()
config_top.depthThresholds.lowerThreshold = 100
config_top.depthThresholds.upperThreshold = 10000
config_top.roi = dai.Rect(topLeft, bottomRight)


# config_left
topLeft = dai.Point2f(0.25, 0.45)
bottomRight = dai.Point2f(0.35, 0.55)

config_left = dai.SpatialLocationCalculatorConfigData()
config_left.depthThresholds.lowerThreshold = 100
config_left.depthThresholds.upperThreshold = 10000
config_left.roi = dai.Rect(topLeft, bottomRight)

# config_bottom
topLeft = dai.Point2f(0.45, 0.65)
bottomRight = dai.Point2f(0.55, 0.75)

config_bottom = dai.SpatialLocationCalculatorConfigData()
config_bottom.depthThresholds.lowerThreshold = 100
config_bottom.depthThresholds.upperThreshold = 10000
config_bottom.roi = dai.Rect(topLeft, bottomRight)

# config_right
topLeft = dai.Point2f(0.65, 0.45)
bottomRight = dai.Point2f(0.75, 0.55)

config_right = dai.SpatialLocationCalculatorConfigData()
config_right.depthThresholds.lowerThreshold = 100
config_right.depthThresholds.upperThreshold = 10000
config_right.roi = dai.Rect(topLeft, bottomRight)

# config_center
topLeft = dai.Point2f(0.45, 0.45)
bottomRight = dai.Point2f(0.55, 0.55)

config_center = dai.SpatialLocationCalculatorConfigData()
config_center.depthThresholds.lowerThreshold = 100
config_center.depthThresholds.upperThreshold = 10000
config_center.roi = dai.Rect(topLeft, bottomRight)

spatialLocationCalculator.initialConfig.setROIs(
    [config_top, config_left, config_center, config_bottom, config_right]
)
stereo.setRuntimeModeSwitch(True)

# Linking
camRgb.video.link(xoutRgb.input)
monoLeft.out.link(stereo.left)
monoRight.out.link(stereo.right)

spatialLocationCalculator.passthroughDepth.link(xoutDepth.input)
stereo.depth.link(spatialLocationCalculator.inputDepth)

spatialLocationCalculator.out.link(xoutSpatialData.input)
xoutDepthConfig.out.link(stereo.inputConfig)


# Connect to device and start pipeline
with dai.Device(pipeline) as device:
    device.setIrLaserDotProjectorBrightness(1200) # in mA, 0..1200
    device.setIrFloodLightBrightness(750) # in mA, 0..1500
    # Output queue will be used to get the depth frames from the outputs defined above
    rgbQueue = device.getOutputQueue(name="rgb", maxSize=4, blocking=False)
    depthQueue = device.getOutputQueue(name="depth", maxSize=4, blocking=False)
    spatialCalcQueue = device.getOutputQueue(
        name="spatialData", maxSize=4, blocking=False
    )
    depthConfigInQueue = device.getInputQueue("DepthConfig")

    color = (255, 255, 255)

    while True:
        # Blocking call, will wait until a new data has arrived
        inDepth = depthQueue.get()
        rgbFrame = rgbQueue.get().getCvFrame()

        depthFrame = inDepth.getFrame()
        depthFrameColor = cv2.normalize(
            depthFrame, None, 255, 0, cv2.NORM_INF, cv2.CV_8UC1
        )
        depthFrameColor = cv2.equalizeHist(depthFrameColor)
        depthFrameColor = cv2.applyColorMap(depthFrameColor, cv2.COLORMAP_JET)

        spatialData = spatialCalcQueue.get().getSpatialLocations()
        for depthData in spatialData:
            roi = depthData.config.roi
            roi = roi.denormalize(
                width=depthFrameColor.shape[1], height=depthFrameColor.shape[0]
            )
            xmin = int(roi.topLeft().x)
            ymin = int(roi.topLeft().y)
            xmax = int(roi.bottomRight().x)
            ymax = int(roi.bottomRight().y)

            # depthMin = depthData.depthMin
            # depthMax = depthData.depthMax

            fontType = cv2.FONT_HERSHEY_TRIPLEX
            cv2.rectangle(
                rgbFrame,
                (xmin, ymin),
                (xmax, ymax),
                color,
                cv2.FONT_HERSHEY_SCRIPT_SIMPLEX,
            )

            cv2.putText(
                rgbFrame,
                f"X: {int(depthData.spatialCoordinates.x)} mm",
                (xmin + 10, ymin + 20),
                fontType,
                0.5,
                color,
            )
            cv2.putText(
                rgbFrame,
                f"Y: {int(depthData.spatialCoordinates.y)} mm",
                (xmin + 10, ymin + 35),
                fontType,
                0.5,
                color,
            )
            cv2.putText(
                rgbFrame,
                f"Z: {int(depthData.spatialCoordinates.z)} mm",
                (xmin + 10, ymin + 50),
                fontType,
                0.5,
                color,
            )

            cv2.rectangle(
                depthFrameColor,
                (xmin, ymin),
                (xmax, ymax),
                color,
                cv2.FONT_HERSHEY_SCRIPT_SIMPLEX,
            )
            cv2.putText(
                depthFrameColor,
                f"X: {int(depthData.spatialCoordinates.x)} mm",
                (xmin + 10, ymin + 20),
                fontType,
                0.5,
                255,
            )
            cv2.putText(
                depthFrameColor,
                f"Y: {int(depthData.spatialCoordinates.y)} mm",
                (xmin + 10, ymin + 35),
                fontType,
                0.5,
                255,
            )
            cv2.putText(
                depthFrameColor,
                f"Z: {int(depthData.spatialCoordinates.z)} mm",
                (xmin + 10, ymin + 50),
                fontType,
                0.5,
                255,
            )
        # Show the frame

        cv2.putText(
            depthFrameColor,
            f"LeftRightCheck = {lr_check}",
            (10, 20),
            fontType,
            0.5,
            (0, 0, 0),
            4,
        )
        cv2.putText(
            depthFrameColor,
            f"Extended = {extended_disparity}",
            (10, 40),
            fontType,
            0.5,
            (0, 0, 0),
            4,
        )
        cv2.putText(
            depthFrameColor,
            f"Subpixel = {subpixel}",
            (10, 60),
            fontType,
            0.5,
            (0, 0, 0),
            4,
        )
        cv2.putText(
            depthFrameColor,
            f"MedianFilter = {median_filter}",
            (10, 80),
            fontType,
            0.5,
            (0, 0, 0),
            4,
        )

        cv2.putText(
            depthFrameColor,
            f"LeftRightCheck = {lr_check}",
            (10, 20),
            fontType,
            0.5,
            (255, 255, 255),
            1,
        )
        cv2.putText(
            depthFrameColor,
            f"Extended = {extended_disparity}",
            (10, 40),
            fontType,
            0.5,
            (255, 255, 255),
            1,
        )
        cv2.putText(
            depthFrameColor,
            f"Subpixel = {subpixel}",
            (10, 60),
            fontType,
            0.5,
            (255, 255, 255),
            1,
        )
        cv2.putText(
            depthFrameColor,
            f"MedianFilter = {median_filter}",
            (10, 80),
            fontType,
            0.5,
            (255, 255, 255),
            1,
        )

        if 1:
            cv2.imshow("depth", cv2.resize(depthFrameColor, (0, 0), fx=0.75, fy=0.75))
            cv2.imshow("rgb", cv2.resize(rgbFrame, (0, 0), fx=0.75, fy=0.75))
        else:
            preview = cv2.hconcat([depthFrameColor, rgbFrame])
            # preview = cv2.resize(preview, (0, 0), fx=0.75, fy=0.75)
            cv2.imshow("preview", preview)

        key = cv2.waitKey(1)
        if key == ord("q"):
            break

        elif key in [ord("0"), ord("3"), ord("5"), ord("7")]:
            median_filter = MedianFilter[chr(key)]
            newConfig = True

        elif key == ord("s"):
            subpixel = not subpixel
            if subpixel:
                extended_disparity = False
            newConfig = True
        elif key == ord("e"):
            extended_disparity = not extended_disparity
            if extended_disparity:
                subpixel = False
            newConfig = True
        elif key == ord("l"):
            lr_check = not lr_check
            newConfig = True
        # elif key == ord("a"):
        #     align =

        if newConfig:
            cfg = dai.StereoDepthConfig()
            cfg.setLeftRightCheck(lr_check)
            cfg.setExtendedDisparity(extended_disparity)
            cfg.setSubpixel(subpixel)
            cfg.setMedianFilter(median_filter)
            # cfg.setDepthAlign(dai.RawStereoDepthConfig.AlgorithmControl.DepthAlign.CENTER)
            depthConfigInQueue.send(cfg)
            newConfig = False
